@@ -1,13 +1,13 @@
-/* 	$Id: strdup.c,v 1.2 1996/01/31 10:02:49 martin Exp $	 */
+/* 	$Id: strdup.c,v 1.2 1996/01/31 10:02:49 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: strdup.c,v 1.2 1996/01/31 10:02:49 martin Exp $";
+static char vcid[] = "$Id: strdup.c,v 1.2 1996/01/31 10:02:49 martin Exp martin $";
 #endif /* lint */
 
 #include <string.h>
 #include <malloc.h>
 
-#if defined(_AIX)
+#if defined(_AIX_)
 char *strdup(char *s)
 #else
 char *strdup(const char *s)
