@@ -35,10 +35,10 @@
 static char sccsid[] = "@(#)commands.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: commands.c,v 1.8 1996/02/01 12:04:24 martin Exp $	 */
+/* 	$Id: commands.c,v 1.9 1997/08/22 10:09:45 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: commands.c,v 1.8 1996/02/01 12:04:24 martin Exp $";
+static char vcid[] = "$Id: commands.c,v 1.9 1997/08/22 10:09:45 martin Exp martin $";
 #endif /* lint */
 
 #include "../config.h"
@@ -91,24 +91,6 @@ static char vcid[] = "$Id: commands.c,v 1.8 1996/02/01 12:04:24 martin Exp $";
 #endif /* !defined(CRAY) && !defined(sysV88) */
 #include <netinet/ip.h>
 
-#ifdef USE_SSL
-
-SSL_CTX *ssl_ctx;
-SSL *ssl_con;
-int ssl_debug_flag=0;
-int ssl_only_flag=0;
-int ssl_active_flag=0;
-int ssl_verify_flag=SSL_VERIFY_NONE;
-int ssl_secure_flag=0;
-int ssl_certsok_flag=0;
-char *cert_file=NULL;
-char *key_file=NULL;
-char *cipher_list=NULL;
-
-extern int display_connect_details();
-
-#endif /* USE_SSL */
-
 #ifndef       MAXHOSTNAMELEN
 #define       MAXHOSTNAMELEN 64
 #endif        /* MAXHOSTNAMELEN */
@@ -1481,6 +1463,10 @@ bye(argc, argv)
     extern int resettermname;
 
     if (connected) {
+#ifdef USE_SSL
+	if (ssl_active_flag)
+	  SSL_shutdown(ssl_con);
+#endif /* USE_SSL */
 	(void) shutdown(net, 2);
 	printf("Connection closed.\n");
 	(void) NetClose(net);
@@ -2322,6 +2308,20 @@ tn(argc, argv)
 	}
 	if (hostp == 0) {
 	    hostp = *argv++;
+
+	    /* support host:port syntax that is now commonly
+	     * used in lots of programs --tjh 
+	     * (though don't disturb the existing IP_OPTIONS 
+	     *  code that has a colon in the syntax)
+	     */
+	    if ((hostp[0]!='@')&&(hostp[0]!='!')) {
+	      char *p;
+
+	      if ((p=strchr(hostp,':'))!=NULL) {
+	        *p='\0';
+		portp=p+1;
+	      }
+	    }
 	    --argc;
 	    continue;
 	}
@@ -2399,9 +2399,33 @@ tn(argc, argv)
 	    if (sp)
 		sin.sin_port = sp->s_port;
 	    else {
+#ifdef USE_SSL
+		/* hardwire in the knowledge of https as I'm
+		 * getting tierd of machines that are configured
+		 * without knowledge of this! --tjh
+		 *
+		 * ... actually I've extended this to include
+		 * all the usual ports that I like going to that
+		 * various machine administrator have been too
+		 * slack to setup
+		 *
+		 */
+		if (strcmp(portp,"http")==0)
+		    sin.sin_port = 80;
+		else if (strcmp(portp,"https")==0)
+		    sin.sin_port = 443;
+		else if (strcmp(portp,"ssl-telnet")==0)
+		    sin.sin_port = 151;
+		/* include the offical IANA port! */
+		else if (strcmp(portp,"telnets")==0)
+		    sin.sin_port=992;
+		else
+#endif /* USE_SSL */
+                {
 		printf("%s: bad port number\n", portp);
 	        setuid(getuid());
 		return 0;
+		}
 	    }
 	} else {
 /*#ifdef STDC_HEADERS
@@ -2483,69 +2507,40 @@ tn(argc, argv)
 	}
 
 #ifdef USE_SSL
-	if (ssl_debug_flag) {
-	    fprintf(stderr,"SSL_DEBUG_FLAG on\n");
+	if (!do_ssleay_init(0)) {
+	  if (bio_err==NULL) {
+	    fflush(stdout);
 	    fflush(stderr);
-	    /* take these two out of the "normal" stream */
-	    SSL_debug("telnet.log");
-	    /*
-	    (void)freopen("telnet.err","w",stderr);
-	    SSL_ERR=stderr;
-	    */
-	    SSL_LOG=SSL_ERR=stderr;
-	} else {
-	    SSL_LOG=SSL_ERR=NULL;
-	}
-
-	/* init things so we will get meaningful error messages
-	 * rather than numbers 
-	 */
-	SSL_load_error_strings();
-
-	ssl_ctx=(SSL_CTX *)SSL_CTX_new();
-	ssl_con=(SSL *)SSL_new(ssl_ctx);
-
-        SSL_set_fd(ssl_con,net);
-        SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
-	if (cipher_list==NULL)
-	    SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
-	else
-	    SSL_set_pref_cipher(ssl_con,cipher_list);
-
-	/* Add in any certificates if you want to here ... */
-	if (cert_file) {
-	    if (!SSL_use_certificate_file(ssl_con, cert_file,
-		X509_FILETYPE_PEM)) {
-		fprintf(stderr,"%s: ",cert_file);
-		ERR_print_errors(stderr);
-		exit(1);
-	    } else {
-		if (!key_file)
-		    key_file = cert_file;
-		if (!SSL_use_RSAPrivateKey_file(ssl_con, key_file,
-		    X509_FILETYPE_PEM)) {
-		    fprintf(stderr,"%s: ", key_file);
-		    ERR_print_errors(stderr);
-		    exit(1);
-		}
-	    }
+	    fprintf(stderr,"do_ssleay_init() failed\n");
+	    ERR_print_errors_fp(stderr);
+	  } else {
+	    BIO_printf(bio_err,"do_ssleay_init() failed\n");
+	    ERR_print_errors(bio_err);
+	  }
+	  exit(1);
 	}
+	
+	/* bind in the network descriptor */
+    	SSL_set_fd(ssl_con,net);
 
+	/* if we are doing raw SSL then start it now ... */
 	if (ssl_only_flag) {
 	    if (!SSL_connect(ssl_con)) {
-                static char errbuf[1024];
+		static char errbuf[1024];
 
-		ERR_print_errors(stderr);
+		ERR_print_errors_fp(stderr);
 		perror("SSL_connect");
 		fflush(stderr);
 
-		return 0;
+		exit(1);
 	    } else {
 		display_connect_details(ssl_con,1);
 		ssl_active_flag=1;
 	    }
 	}
+
 #endif /* USE_SSL */
+
 	connected++;
 #if	defined(AUTHENTICATION) || defined(ENCRYPTION)
 	auth_encrypt_connect(connected);
