@@ -47,14 +47,17 @@ static char copyright[] =
 static char sccsid[] = "@(#)telnetd.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: telnetd.c,v 1.8 1996/06/13 14:20:12 martin Exp $	 */
+/* 	$Id: telnetd.c,v 1.9 1997/08/22 10:20:45 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: telnetd.c,v 1.8 1996/06/13 14:20:12 martin Exp $";
+static char vcid[] = "$Id: telnetd.c,v 1.9 1997/08/22 10:20:45 martin Exp martin $";
 #endif /* lint */
 
 #include "telnetd.h"
 #include "pathnames.h"
+#ifdef HAVE_POLL_H
+#include <poll.h>
+#endif
 
 #include "../lib/libtelnet/misc-proto.h"
 
@@ -150,6 +153,10 @@ char	ptyibuf2[BUFSIZ];
 
 #endif /* ! STREAMPTY */
 
+#ifdef USE_SSL 
+static char cert_filepath[1024];
+#endif /* USE_SSL */
+
 int	hostinfo = 1;			/* do we print login banner? */
 
 #ifdef	CRAY
@@ -203,28 +210,6 @@ char valid_opts[] = {
 	'\0'
 };
 
-#ifdef USE_SSL
-
-/*
-#include "ssl_err.h"
-*/
-  
-SSL_CTX *ssl_ctx;
-SSL *ssl_con;
-int ssl_debug_flag=0;
-int ssl_only_flag=0;
-int ssl_active_flag=0;
-int ssl_secure_flag=0;
-int ssl_certsok_flag=0;
-int ssl_verify_flag=SSL_VERIFY_NONE;
-char ssl_file_path[1024];    /* don't look at that nasty value to the left */
-
-static char *my_ssl_key_file=NULL;
-static char *my_ssl_cert_file=NULL;
-char *cipher_list;
-
-#endif /* USE_SSL */
-
         void
 doit(struct sockaddr_in *who);
 
@@ -251,6 +236,17 @@ main(argc, argv)
 
 	progname = *argv;
 
+#ifdef USE_SSL
+	/* we need to know the fullpath to the location of the
+	 * certificate that we will be running with as we cannot
+	 * be sure of the cwd when we are launched
+	 */
+	sprintf(cert_filepath,"%s/%s",X509_get_default_cert_dir(),
+	        "telnetd.pem");
+	ssl_cert_file=cert_filepath;
+	ssl_key_file=NULL;
+#endif /* USE_SSL */
+
 #ifdef CRAY
 	/*
 	 * Get number of pty's before trying to process options,
@@ -264,32 +260,63 @@ main(argc, argv)
 
 #ifdef USE_SSL
                 case 'z':
+		        { 
+			char *origopt;
+
+			origopt=strdup(optarg);
+			optarg=strtok(origopt,",");
+
+			while(optarg!=NULL) {
+
 		        if (strcmp(optarg, "debug") == 0 ) {
 			    ssl_debug_flag=1;
-			}
-		        if (strcmp(optarg, "ssl") == 0 ) {
-			    ssl_only_flag=1;
-			}
-		        if (strcmp(optarg, "certsok") == 0 ) {
-			    ssl_certsok_flag=1;
-			}
-		        if (strcmp(optarg, "secure") == 0 ) {
-			    ssl_secure_flag=1;
-			}
-		        if (strncmp(optarg, "verify=", strlen("verify=")) == 0 ) {
-			    ssl_verify_flag=atoi(optarg+strlen("verify="));
+			} else if (strcmp(optarg, "ssl") == 0 ) {
+  			    ssl_only_flag=1;
+			} else if (strcmp(optarg, "certsok") == 0 ) {
+  			    ssl_certsok_flag=1;
+			} else if ( (strcmp(optarg, "!ssl") == 0) ||
+		             (strcmp(optarg, "nossl") == 0) ) {
+			    /* we may want to switch SSL negotiation off
+			     * for testing or other reasons 
+			     */
+			    ssl_disabled_flag=1;
+			} else if (strcmp(optarg, "certrequired") == 0 ) {
+			    ssl_cert_required=1;
+			} else if (strcmp(optarg, "secure") == 0 ) {
+  			    ssl_secure_flag=1;
+			} else if (strncmp(optarg, "verify=", 
+			                strlen("verify=")) == 0 ) {
+  			    ssl_verify_flag=atoi(optarg+strlen("verify="));
+			} else if (strncmp(optarg, "cert=", 
+			                strlen("cert=")) == 0 ) {
+			    ssl_cert_file=optarg+strlen("cert=");
+			} else if (strncmp(optarg, "key=", 
+			                strlen("key=")) == 0 ) {
+			    ssl_key_file=optarg+strlen("key=");
+			} else if (strncmp(optarg,"cipher=",
+			                strlen("cipher="))==0) {
+			    ssl_cipher_list=optarg+strlen("cipher=");
+			} else {
+			    /* report when we are given rubbish so that
+			     * if the user makes a mistake they have to
+			     * correct it!
+			     */
+			    fprintf(stderr,"Unknown SSL option %s\n",optarg);
+			    fflush(stderr);
+			    exit(1);
+  			}
 
-			}
-		        if (strncmp(optarg, "cert=", strlen("cert=")) == 0 ) {
-			    my_ssl_cert_file=optarg+strlen("cert=");
-			}
-		        if (strncmp(optarg, "key=", strlen("key=")) == 0 ) {
-			    my_ssl_key_file=optarg+strlen("key=");
-			}
+			/* get the next one ... */
+                        optarg=strtok(NULL,",");
 
-		        if (strncmp(optarg,"cipher=",sizeof("cipher"))==0) {
-			    cipher_list=optarg+sizeof("cipher");
-			}
+  			}
+  
+			/*
+			if (origopt!=NULL)
+			    free(origopt);
+			*/
+
+  			}
 			break;
 #endif /* USE_SSL */
 
@@ -476,37 +503,46 @@ main(argc, argv)
 	}
 
 #ifdef USE_SSL
-        {
-	  /*	    int i;*/
 
-	    /* if we are not running in debug then any error
-	     * stuff from SSL debug *must* not go down
-	     * the socket (which 0,1,2 are all pointing to by
-	     * default)
+        if (ssl_secure_flag || ssl_cert_required) {
+	    /* in secure mode we *must* switch on the base level
+	     * verify checking otherwise we cannot abort connections
+	     * at the right place!
 	     */
-	    if (ssl_debug_flag) {
-		if (!debug) {
-		    (void)freopen("telnetd.err","w",stderr);
-		    SSL_ERR=stderr;
-		}
-		SSL_debug("telnetd.log");
-	    } else {
-		/* disable all the debug and trace */
-		SSL_LOG=SSL_ERR=NULL;
-	    }
- 	    SSL_load_error_strings();
-	    ssl_ctx=(SSL_CTX *)SSL_CTX_new();
-
-            if (!X509_set_default_verify_paths(ssl_ctx->cert)) {
-		fprintf(stderr,"telnetd: cannot load certificates via X509_set_default_verify_paths\n");
-		fflush(stderr);
-		ERR_print_errors(stderr);
-		exit(1);
-	    }
-
+	    if (ssl_verify_flag==0)
+		ssl_verify_flag=1;
 	}
-#endif /* USE_SSL */
+  
+	/* if we are not running in debug then any error
+	 * stuff from SSL debug *must* not go down
+	 * the socket (which 0,1,2 are all pointing to by
+	 * default)
+	 */
+	if (ssl_debug_flag)
+	    ssl_log_file="/telnetd.log";
 
+	if (!do_ssleay_init(1)) {
+	  if (bio_err!=NULL) {
+	    BIO_printf(bio_err,"do_ssleay_init() failed\n");
+	    ERR_print_errors(bio_err);
+	  } else {
+	    fflush(stderr);
+	    fprintf(stderr,"do_ssleay_init() failed\n");
+	    ERR_print_errors_fp(stderr);
+	  }
+	  exit(1);
+	}
+  
+	if (ssl_debug_flag) {
+	  int i;
+	  
+	  BIO_printf(bio_err,"secure %d certrequired %d verify %d\n",
+	      ssl_secure_flag,ssl_cert_required,ssl_verify_flag);
+	  for(i=0;i<argc;i++)
+	      BIO_printf(bio_err,"argv[%d]=\"%s\"\n",i,argv[i]);
+  	}
+  
+#endif /* USE_SSL */
 
 	argc -= optind;
 	argv += optind;
@@ -662,42 +698,8 @@ main(argc, argv)
 
 #ifdef USE_SSL
         /* do the SSL stuff now ... before we play with pty's */
-	ssl_con=(SSL *)SSL_new(ssl_ctx);
 	SSL_set_fd(ssl_con,0);
 
-        /* just use telnetd.pem for public and private and don't
-	 * encrypt it with a pass phrase 
-	 */
-	sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
-	        "telnetd.pem");
-	if (SSL_use_RSAPrivateKey_file(ssl_con,
-	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file,
-		X509_FILETYPE_PEM)<=0) {
-	    fprintf(stderr,"SSL_use_RSAPrivateKey_file(%s) error ",
-	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file);
-	    ERR_print_errors(stderr);
-	    fflush(stderr);
-	    sleep(5);
-	    _exit(1);
-	}
-
-	sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
-	        "telnetd.pem");
-	if (SSL_use_certificate_file(ssl_con,
-	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file,
-	        X509_FILETYPE_PEM)<=0) {
-	    fprintf(stderr,"SSL_use_certificate_file(%s) error ",
-	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file);
-	    ERR_print_errors(stderr);
-	    fflush(stderr);
-	    sleep(5);
-	    _exit(1);
-	}
-
-	/* if we gave a cipher list then ... */
-          if (cipher_list!=NULL)
-	    SSL_set_pref_cipher(ssl_con,cipher_list);
-
 	if (ssl_only_flag) {
 	    /* hmm ... only when running talking to things like
 	     * https servers should we hit this code and then
@@ -713,20 +715,23 @@ main(argc, argv)
 
 		syslog(LOG_WARNING, errbuf);
 
-		fprintf(stderr,errbuf);
-		fflush(stderr);
+		BIO_printf(bio_err,errbuf);
 
+		/* go to sleep to make sure we are noticed */
 		sleep(10);
 		SSL_free(ssl_con);
+
 		_exit(1);
 	    } else {
 		ssl_active_flag=1;
 	    }
 	}
-
 #endif /* USE_SSL */
 
 	net = 0;
+#if defined(SOLARIS)
+        close(1);close(2);
+#endif
 	doit(&from);
 	/* NOTREACHED */
 	return 0;
@@ -863,6 +868,11 @@ getterminaltype(name)
 	    }
 #endif
 
+            if (ssl_debug_flag) {
+		fprintf(stderr,"[SSL required - connection rejected]");
+		fflush(stderr);
+	    }
+
 	    fatal(net,"[SSL required - connection rejected]");
 
 	}
@@ -1194,6 +1204,45 @@ Xterm_output(ibufp, obuf, icountp, ocount)
 #define	term_output	Xterm_output
 #endif	/* defined(CRAY2) && defined(UNICOS5) && defined(UNICOS50) */
 
+#ifdef hpux_bla
+int select(int nfds,
+		  fd_set *readfds,
+		  fd_set *writefds,
+		  fd_set *errorfds,
+		  struct timeval *timeout)
+{
+  struct pollfd *fds;
+  int fd, ret;
+
+  fds = malloc(nfds * sizeof(struct pollfd));
+  for (fd = 0; fd < nfds; fd++)
+    {
+      fds[fd].fd = fd;
+      fds[fd].events =
+	(readfds == NULL ? 0 : (FD_ISSET(fd, readfds) ? POLLIN : 0)) |
+	(writefds == NULL ? 0 : (FD_ISSET(fd, writefds) ? POLLOUT : 0)) | 
+	(errorfds == NULL ? 0 : (FD_ISSET(fd, errorfds) ? POLLPRI : 0));
+    }
+  ret = poll(fds, nfds,
+	     timeout == NULL ? -1 : timeout->tv_usec + 1000 * timeout->tv_sec);
+  if (readfds != NULL) FD_ZERO(readfds);
+  if (writefds != NULL) FD_ZERO(writefds);
+  if (errorfds != NULL) FD_ZERO(errorfds);
+  for (fd = 0; fd < nfds; fd++)
+    {
+      if ((readfds != NULL) && (fds[fd].revents & POLLIN))
+	FD_SET(fd, readfds);
+      if ((writefds != NULL) && (fds[fd].revents & POLLOUT))
+	FD_SET(fd, writefds);
+      if ((errorfds != NULL) && (fds[fd].revents & POLLPRI))
+	FD_SET(fd, errorfds);
+    }
+  free(fds);
+  
+  return ret;
+}
+#endif
+
 /*
  * Main loop.  Select from pty and network, and
  * hand data to telnet receiver finite state machine.
@@ -1343,8 +1392,12 @@ telnet(f, p, host)
 #ifndef USE_SSL
 	(void) ioctl(f, FIONBIO, (char *)&on);
 #endif /* !USE_SSL */
-
+#ifdef hpux
+	/*	(void) ioctl(p, FIOSNBIO, (char *)&on);*/
+#else
 	(void) ioctl(p, FIONBIO, (char *)&on);
+#endif
+
 #if	defined(CRAY2) && defined(UNICOS5)
 	init_termdriver(f, p, interrupt, sendbrk);
 #endif
@@ -1453,6 +1506,10 @@ telnet(f, p, host)
 	for (;;) {
 		fd_set ibits, obits, xbits;
 		register int c;
+		struct timeval tv;
+#ifdef hpux_bla
+		struct pollfd fds[2];
+#endif
 
 		if (ncc < 0 && pcc < 0)
 			break;
@@ -1481,8 +1538,38 @@ telnet(f, p, host)
 		if (!SYNCHing) {
 			FD_SET(f, &xbits);
 		}
-		if ((c = select(16, &ibits, &obits, &xbits,
-						(struct timeval *)0)) < 1) {
+                errno = 0;
+                {
+		  char buff[255];
+		  int res;
+		  /*		  res = read(p, buff, 2);*/
+                }
+                errno = 0;
+		tv.tv_sec = 60;
+		tv.tv_usec = 0;
+
+#ifdef hpux_bla
+		fds[0].fd = f;
+		fds[0].events = (FD_ISSET(f, &ibits) ? POLLIN : 0) | (FD_ISSET(f, &obits) ? POLLOUT : 0);
+		fds[1].fd = p;
+		fds[1].events = (FD_ISSET(p, &ibits) ? POLLIN : 0) | (FD_ISSET(p, &obits) ? POLLOUT : 0);
+		if ((c = poll(fds, 2, 60000)) < 1) {
+		  sleep(5);
+		  continue;
+		}
+		FD_ZERO(&ibits);
+		FD_ZERO(&obits);
+		if (fds[0].revents & POLLIN)
+		  FD_SET(f, &ibits);
+		if (fds[0].revents & POLLOUT)
+		  FD_SET(f, &obits);
+		if (fds[1].revents & POLLIN)
+		  FD_SET(p, &ibits);
+		if (fds[1].revents & POLLOUT)
+		  FD_SET(p, &obits);
+#else
+		if ((c = select(32, &ibits, &obits, &xbits,
+						(struct timeval *)&tv)) < 1) {
 			if (c == -1) {
 				if (errno == EINTR) {
 					continue;
@@ -1491,6 +1578,7 @@ telnet(f, p, host)
 			sleep(5);
 			continue;
 		}
+#endif
 
 		/*
 		 * Any urgent data?
@@ -1573,6 +1661,10 @@ telnet(f, p, host)
 #endif /* USE_SSL */
 		    ncc = read(net, netibuf, sizeof (netibuf));
 #endif	/* !defined(SO_OOBINLINE)) */
+		    if (debug) {
+		      fprintf(stderr,"net_read: %d %d\n",ncc,errno);
+		      fflush(stderr);
+		    }
 		    if (ncc < 0 && errno == EWOULDBLOCK)
 			ncc = 0;
 		    else {
@@ -1596,6 +1688,18 @@ telnet(f, p, host)
 #else
 			pcc = readstream(p, ptyibuf, BUFSIZ);
 #endif
+			if (debug) {
+				fprintf(stderr,"pty_read: %d %d\n",pcc,errno);
+				fflush(stderr);
+			}
+#ifdef LINUX
+			/* EIO on pty means closed under Linux 2.x kernels
+			 * at least ... so we clear it so it isn't treated
+			 * as EWOULDBLOCK by the code below! --tjh
+			 */
+			if (errno==EIO)
+				errno=0;
+#endif /* LINUX */
 			/*
 			 * On some systems, if we try to read something
 			 * off the master side before the slave side is
@@ -1699,11 +1803,15 @@ telnet(f, p, host)
 #endif	/* defined(CRAY2) && defined(UNICOS5) */
 
 		if (FD_ISSET(f, &obits) && (nfrontp - nbackp) > 0)
+		  {
 			netflush();
+		  }
 		if (ncc > 0)
 			telrcv();
 		if (FD_ISSET(p, &obits) && (pfrontp - pbackp) > 0)
+		  {
 			ptyflush();
+		  }
 	}
 	cleanup(0);
 }  /* end of telnet */
@@ -1777,6 +1885,7 @@ int readstream(p, ibuf, bufsize)
 			vstart = tsp->c_cc[VSTART];
 			ixon = tsp->c_iflag & IXON;
 			break;
+#ifndef hpux
 		case TCSETA:
 		case TCSETAW:
 		case TCSETAF:
@@ -1785,6 +1894,7 @@ int readstream(p, ibuf, bufsize)
 			vstart = tp->c_cc[VSTART];
 			ixon = tp->c_iflag & IXON;      
 			break;
+#endif
 		default:
 			errno = EAGAIN;
 			return(-1);
